from setuptools import setup, find_packages

setup(
    name="team3_pobl_chassis",
    version="1.0.1",
    packages=find_packages(),
    install_requires=[],
    author="macc-team3",
    author_email="gorka.etxebarria@alumni.mondragon.edu",
    description="A reusable library for microservices",
)
